package ru.mkb.corp.monmod.service.client;

import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.View;
import com.haulmont.cuba.core.global.ViewBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import ru.mkb.corp.monmod.core.CuratorListToCuratorMapper;
import ru.mkb.corp.monmod.core.SiebelService;
import ru.mkb.corp.monmod.entity.counterparty.Client;
import ru.mkb.corp.monmod.entity.counterparty.Curator;
import ru.mkb.corp.monmod.entity.counterparty.Holding;
import ru.mkb.corp.monmod.entity.monitoring.PvLevel;
import ru.mkb.siebelservice.crpallprognozwf.MKBCRPHoldingInfoOut14;
import ru.mkb.siebelservice.crpallprognozwf.MKBCRPHoldingLinksIn14;
import ru.mkb.siebelservice.crpallprognozwf.MKBCRPHoldingLinksIn14TopElmt;

import javax.inject.Inject;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service(CrmHoldingService.NAME)
@Slf4j
public class CrmHoldingServiceBean implements CrmHoldingService {

    @Inject
    private DataManager dataManager;

    @Inject
    private SiebelService siebelService;

    @Inject
    private CrmClientService createClientService;

    @Inject
    private CuratorListToCuratorMapper curatorListToCuratorMapper;


    @Override
    public Holding getOrCreateByCrmCode(String crmCode) {
        return dataManager.load(Holding.class)
                .query("e.crmId = :crmCode")
                .parameter("crmCode", crmCode)
                .optional()
                .map(this::updateFromCrm)
                .orElseGet(() -> createByCrmCode(crmCode));
    }

    @Override
    public Holding createByCrmCode(String crmCode) {
        return createByMKBCRPHoldingInfo(getMkbcrpHoldingInfoByCrmCode(crmCode));
    }

    private MKBCRPHoldingInfoOut14 getMkbcrpHoldingInfoByCrmCode(String crmCode) {
        MKBCRPHoldingLinksIn14TopElmt request = new MKBCRPHoldingLinksIn14TopElmt();
        MKBCRPHoldingLinksIn14 value = new MKBCRPHoldingLinksIn14();
        value.setSiebelID(crmCode);
        request.setMKBCRPHoldingLinksIn14(value);
        return siebelService.getHoldingInfoService().sblesc14(request).getMKBCRPHoldingInfoOut14();
    }

    private Holding createByMKBCRPHoldingInfo(MKBCRPHoldingInfoOut14 info) {
        Holding holding = dataManager.create(Holding.class);
        Holding result = updateHolding(holding, info);
        return result;
    }

    private Holding updateHolding(Holding holding, MKBCRPHoldingInfoOut14 info) {
        holding.setName(info.getHoldingName());
        holding.setAbsId(info.getHoldingID());
        holding.setCrmId(info.getSiebelID());
        holding.setSiebelRowId(info.getSiebelRowID());
        holding.setSiebelLastUpdateDate(LocalDateTime.now());
        holding.setCurator(getCurator(info));
        holding.setPvLevel(Optional.ofNullable(Integer.parseInt(info.getHldLevelAttention())).map(PvLevel::fromId).orElse(null));
        return dataManager.commit(holding);
    }

    private List<Client> updateHoldingStructureByMKBCRPHoldingInfo(Holding holding, MKBCRPHoldingInfoOut14 info) {
        List<Client> ourClients = dataManager.load(Client.class)
                .query("e.holding = ?1", holding)
                .list();
        List<String> ourCrmCodes = ourClients.stream().map(Client::getCrmId).collect(Collectors.toList());
        List<Client> insertClients = info
                .getAccountList()
                .stream()
                .filter(accountList -> !ourCrmCodes.contains(accountList.getSiebelID()))
                .map(accountList -> createClientService.getOrCreateByCrmCode(accountList.getSiebelID()))
                .collect(Collectors.toList());
        return Stream.concat(ourClients.stream(), insertClients.stream()).collect(Collectors.toList());
    }

    @Override
    public Holding updateFromCrm(Holding holding) {
        Holding reload = dataManager.reload(holding,
                ViewBuilder.of(Holding.class)
                        .addView(View.LOCAL)
                        .add("curator", View.LOCAL)
                        .build());
        MKBCRPHoldingInfoOut14 mkbcrpHoldingInfoByCrmCode = getMkbcrpHoldingInfoByCrmCode(reload.getCrmId());
        if (needUpdate(reload, mkbcrpHoldingInfoByCrmCode)) {
            Holding result = updateHolding(reload, mkbcrpHoldingInfoByCrmCode);
            return result;
        } else {
            return holding;
        }
    }

    private boolean needUpdate(Holding holding, MKBCRPHoldingInfoOut14 holdingInfo) {
        if (holding.getSiebelLastUpdateDate() != null) {
            if (StringUtils.isEmpty(holdingInfo.getLastUpdate())) {
                return false;
            } else {
                try {
                    return holding.getSiebelLastUpdateDate().compareTo(LocalDateTime.parse(holdingInfo.getLastUpdate(), SiebelService.DATE_TIME_FORMATTER)) < 0;
                } catch (Exception e) {
                    log.error("Error while getting MKBCRPHoldingInfoOut14.lastUpdate.", e);
                    return false;
                }
            }
        } else {
            return true;
        }
    }

    private Curator getCurator(MKBCRPHoldingInfoOut14 holdingInfo) {
        return getCurator(curatorListToCuratorMapper.createCuratorFromCuratorList(holdingInfo.getCuratorList()));
    }

    private Curator getCurator(Curator theirCurator) {
        if (theirCurator == null) {
            return null;
        }
        return dataManager.load(Curator.class)
                .query("e.xusersid = ?1", theirCurator.getXusersid())
                .optional()
                .orElseGet(() -> dataManager.commit(theirCurator));
    }
}